# Foundation Auth

[![lerna](https://img.shields.io/badge/maintained%20with-lerna-cc00ff.svg)](https://lerna.js.org/)
[![TypeScript](https://img.shields.io/badge/%3C%2F%3E-TypeScript-%230074c1.svg)](https://www.typescriptlang.org/)

`foundation-auth` provides several exports to promote reuse and extensibility at various levels. These include but are
not limited to:

* `Auth`: The fully featured authentication micro frontend.
* `User`: The user singleton, which always exists in either an anonymous or authenticated mode.
* `AuthStore`: An auth based `foundation-store`.
* `AuthMachine`: An auth based `foundation-state-machine` covering the various auth workflows.
* `AuthMessageMapper`: The auth message dto to entity mapper.
* `LogoutUtility`: A logout utility.
* Various UI components / screens.

In most cases, developers simply import both the `User` and `Auth` micro frontend, and use the `configure` export token
to tailor the micro frontend to their specific needs in their router config. However, developers can also use the
exports provided to build their own authentication frontends or workflows. A set of pre-defined custom events are used
to interact with the `AuthStore` and `AuthMachine`, and developers can provide custom implementations of parts of the
machine's workflow.

## Basic setup instructions
These examples are based on a host application using [FAST](https://www.fast.design/) and the
[FASTRouter](https://www.npmjs.com/package/@microsoft/fast-router). However, setup in the frontend and router of your
choice should be quite similar. 

### Get User reference
User is a singleton. You can access it via the DI container.
```ts
import { User } from '@genesislcap/foundation-auth/user';
...
@User private user: User
```

...or via a utility if your app is not using the DI system.
```ts
import { getFromContainer, User } from '@genesislcap/foundation-auth/user';
...
private user: User = getFromContainer();
```

User properties are decorated with `@observable`, meaning in FAST terms it's reactive, and you can bind to properties
directly. For example, `this.user.isAuthenticated`. However, you can also monitor the user's authentication status using
an event listener.

```ts
import type { UserAuthenticationChangeEvent } from '@genesislcap/foundation-auth/user';
...
this.user.addAuthenticationChangeListener((event: UserAuthenticationChangeEvent) => {
  console.log('User authenticated?', event.detail);
});
```

### Set up a route to host the micro frontend
Here we are lazy loading the micro frontend when the route is accessed.
```ts
{
  title: 'Login',
  name: 'login',
  path: 'login', // < at what path should the micro frontend be accessed
  element: async () => {
    const { Auth, configure } = await import('@genesislcap/foundation-auth');
    configure({
      omitRoutes: ['request-account'],
      hostPath: 'login', // < important to match the route path
      defaultRedirectUrl: 'my-dashboard',
      logo: myLogo,
      background: myBG,
    });
    return Auth;
  },
  childRouters: true,
  settings: { public: true },
  layout: authLayout,
}
```

### Checking the user authentication status during routing
Here in our `FASTRouter`'s `NavigationContributor`, we check the user status as part of the routing flow.

```ts
this.contributors.push({
  navigate: async (phase) => {
    const settings = phase.route.settings;
    /**
     * Don't block public routes or any routes when authenticated
     */
    if (settings?.public || this.user.isAuthenticated) {
      return;
    }
    /**
     * Otherwise route them to login
     */
    phase.cancel(() => {
      this.user.trackPath();
      Route.name.replace(phase.router, 'login');
    });
  },
});
```

### Establishing a Websocket Connection
Post login you can import the `SOCKET_URL` from auth and connect to the backend.

```ts
import { SOCKET_URL } from '@genesislcap/foundation-auth/env';
...
@Connect private connect: Connect,
...
this.connect.connect(SOCKET_URL);
```

## [API Docs](./docs/api/index.md)

## License

Note: this project provides front-end dependencies and uses licensed components listed in the next section; thus, licenses for those components are required during development. Contact [Genesis Global](https://genesis.global/contact-us/) for more details.

### Licensed components
Genesis low-code platform

# Docs to aid collaboration during development (Will be removed)

1. Get VPN up and ensure `public-foundation-hardened1` is running, then run:

```shell
npm run dev:withVars
```

This will load the .envs. There's work to do in `genx` to load these by default, so we need `lerna` to do it for now.

If you're testing as part of an app, there needs to be a .env file in that root, and it should contain the following variables:

```dotenv
# There should be nothing sensetive here!

# HTTP
GENX_LOGIN_URL=https://public-foundation-hardened1/sm/event-login-auth
GENX_LOGIN_REFRESH_URL=https://public-foundation-hardened1/sm/event-login-auth
GENX_LOGIN_DETAILS_URL=https://public-foundation-hardened1/sm/event-login-details
GENX_LOGOUT_URL=https://public-foundation-hardened1/sm/event-logout
GENX_CHANGE_PASSWORD_URL=https://public-foundation-hardened1/sm/event-change-user-password
GENX_FORGOT_PASSWORD_URL=https://public-foundation-hardened1/sm/event-self-service-password-reset
GENX_RESET_PASSWORD_URL=https://public-foundation-hardened1/sm/event-password-reset-action

# Socket
GENX_SOCKET_URL=wss://public-foundation-hardened1/gwf/
```

You will see warnings in the console if any are unset.

2. Autofill the form with JohnDoe and the usual password.

3. Press Login button or use the shortcut links on the login screen's footer to
trigger machine events to test various, ie. when you already have a cookie, or logout to clear it, or open socket etc.

4. You may need to visit the backend url and accept the bad cert before the above.

5. Socket opening and retry logic is still in the socket service, so happens outside of the state machine. We should be getting a status code back for disallowing the upgrade with bad creds, vs just can't open of unknown reason, but not sure that's working. Raised it. Need to double-check.

6. Other gotchas like the previous point exist.

