import { ElementStyles } from '@microsoft/fast-element';
/**
 * @public
 */
export declare const defaultOrganisation: string;
/**
 * @public
 */
export declare const defaultUser: string;
/**
 * @public
 */
export declare const defaultPassword: string;
/**
 * @public
 */
export declare const defaultApiHost: string;
/**
 * @public
 */
export declare const defaultSocketExt: string;
/**
 * @public
 */
export declare const hostEnv: string;
/**
 * @public
 */
export declare const hostUrl: string;
/**
 * @public
 */
export declare const Routes: {
    readonly login: "login";
    readonly 'request-account': "request-account";
    readonly 'reset-password': "reset-password";
    readonly 'forgot-password': "forgot-password";
    readonly verify: "verify";
    readonly protected: "protected";
    readonly 'not-found': "not-found";
};
/**
 * @public
 */
export type Routes = (typeof Routes)[keyof typeof Routes];
/**
 * A utility to check if a pathname is a route of this micro frontend.
 *
 * @param pathname - A pathname string to check, typically location.pathname, which is used when a value is omitted.
 *
 * @public
 * @privateRemarks
 * I've just put this util in for now given there's some app level direction code in place which needs review.
 */
export declare function isMFRoute(pathname?: string): boolean;
/**
 * A utility to get a sub route path taking the host path into account.
 *
 * @param routeName - The route name.
 * @param hostPath - The host path.
 *
 * @public
 */
export declare function getSubRoutePath(routeName: Routes | string, hostPath?: string): string;
/**
 * @public
 * Could be expanded to allow field ViewTemplates.
 */
export type FieldConfig = {
    label: string;
    /**
     * @privateRemarks
     * Added for now to apply defaults, but I'm not convinced we should do this. Could be made reactive after upgrade.
     */
    value?: any;
    /**
     * @remarks
     * We expect fields to be required by default, but you can override to make them optional. Organisation for example.
     */
    required?: boolean;
    /**
     * @remarks
     * A regular expression the control's value should match. Please note the email field has its own built in pattern.
     */
    pattern?: string;
    /**
     * @remarks
     * Used in conjunction with the pattern to explain the conditions.
     */
    title?: string;
};
/**
 * @public
 * @privateRemarks
 * Using null to exclude the organisation field rather than boolean, as it affords cleaner value lookups in the templates.
 */
export type FieldConfigMap = {
    organisation: FieldConfig | null;
    username: FieldConfig;
    email: FieldConfig;
    password: FieldConfig;
};
/**
 * @public
 */
export type SSOConfig = {
    /**
     * Optionally toggle the SSO journey on/off by default.
     */
    toggled?: boolean;
    /**
     * The identity providers path under the current host.
     *
     * @example
     * ```ts
     * identityProvidersPath: 'sso/list'
     * ```
     */
    identityProvidersPath: string;
};
/**
 * AuthConfig DI interface.
 *
 * @public
 */
export interface AuthConfig {
    /**
     * Login automatically or require user to click a login button after a connection is established.
     *
     * @remarks
     * Needs to manually be enabled in the NavigationContributor.
     */
    autoAuth: boolean;
    /**
     * Logo styles.
     *
     * @remarks
     * Providing `null` will hide the logo. Use content: url(...); to set source.
     *
     * @example Setting a custom logo
     * ```ts
     * logo: css`
     *     content: url(${customLogoImport});
     * `,
     * ```
     */
    logo: ElementStyles | null;
    /**
     * Background styles.
     *
     * @remarks
     * See {@link https://developer.mozilla.org/en-US/docs/Web/CSS/background} for more information.
     *
     * @example Setting a custom background image
     * ```ts
     * background: css`
     *     :host {
     *         background-image: url(${customBGImport});
     *     }
     * `,
     * ```
     */
    background: ElementStyles;
    /**
     * Omit any of the internal routes except 'login' and 'not-found'.
     *
     * @remarks
     * Routes specified here will not be configured nor will any internal cross-links to them be displayed.
     *
     * @example
     * ```ts
     * omitRoutes: ['request-account'],
     * ```
     */
    omitRoutes: Exclude<Routes, 'login' | 'not-found'>[];
    /**
     * Map of configuration for each of the primary form fields.
     *
     * @remarks
     * Use to drive inclusion / exclusion, tailor labels etc.
     */
    fields: FieldConfigMap;
    /**
     * The `path` of the micro frontend as defined in the parent / host route.
     *
     * @remarks
     * Used to help resolve internal route links. Defaults to an empty string.
     */
    hostPath: string;
    /**
     * The default URL to route the user to after a successful login.
     *
     * @remarks
     * Session.returnUrl set via Session.captureReturnUrl() trumps this value.
     */
    defaultRedirectUrl: string;
    /**
     * Omit certain return URLs which the session service may have captured.
     *
     * @example
     * ```ts
     * omitRedirectUrls: ['/not-found'],
     * ```
     */
    omitRedirectUrls: string[];
    /**
     * The routing method called post login and logout.
     *
     * @remarks
     * By default, we use the FASTRouter which internally invokes the native History APIs. The post login url is the last
     * url visited by the user by default or undefined. The post logout url is a slash `/` by default. When overriding
     * this, you should include some protection and may want to ignore when the URL is equal to the current url.
     *
     * @param url - The url to route to or undefined.
     */
    routeToUrl: (url: string | undefined) => void;
    /**
     * SSO configuration.
     *
     * @remarks
     * Providing `null` excludes the SSO journey option completely.
     */
    sso: SSOConfig | null;
    /**
     * Display arbitrary version information.
     *
     * @remarks
     * Applications may choose to set this to display their Application release version on the login form.
     *
     * @example
     * ```ts
     * versionInformation: 'Version 3.0.0',
     * ```
     */
    versionInformation?: string;
    /**
     * Feedback delay.
     *
     * @remarks
     * The amount of seconds to display user feedback messages, such as "check your inbox" etc. Defaults to 3 seconds.
     */
    feedbackDelay?: number;
}
/**
 * Default AuthConfig DI implementation.
 * @public
 */
export declare const defaultAuthConfig: AuthConfig;
/**
 * AuthConfig DI key.
 *
 * @internal
 * @privateRemarks
 * Marked as internal to stop api-extractor becoming confused cross-linking tokens with the same name.
 */
export declare const AuthConfig: import("@microsoft/fast-foundation").InterfaceSymbol<AuthConfig>;
/**
 * Configure the auth micro frontend settings.
 *
 * @remarks
 * This is primarily for general settings, rather than re-defining the element, static templating / style changes.
 *
 * @param config - A partial AuthConfig.
 *
 * @example Configuring auth on lazy load
 * ```ts
 * name: 'login',
 * path: 'login',
 * title: 'Login',
 * element: async () => {
 *   const { configure, define, defaultAuthConfig } = await import('@genesislcap/foundation-auth');
 *   configure({
 *     omitRoutes: ['request-account'],
 *     fields: {
 *       ...defaultAuthConfig.fields,
 *       organisation: {
 *         label: 'CompID',
 *       },
 *     },
 *     hostPath: 'login',
 *     defaultRedirectUrl: 'dashboard',
 *     logo: authLogo,
 *     background: authBG,
 *   });
 *   // You can import and return `Auth` directly here or re-define it completely via `define`.
 *   return define({
 *     name: `nexus-login`,
 *   });
 * },
 * layout: authLayout,
 * settings: { public: true },
 * childRouters: true,
 * ```
 *
 * @public
 * @privateRemarks
 * We can tailor the spreading logic after we put it to use. Keeping it simple for now.
 */
export declare function configure(config: Partial<AuthConfig>): void;
//# sourceMappingURL=config.d.ts.map