import { Listener } from '@genesislcap/foundation-utils';
import type { Details, LoginAck } from '../entity/loginAck';
import { UserConfig } from './config';
/**
 * UserAuthenticationChangeType.
 * @public
 */
export declare const UserAuthenticationChangeType = "user-authentication-change";
/**
 * UserAuthenticationChangeEvent.
 * @remarks
 * Detail will either be `true` or `false` to indicate authentication status.
 * @public
 */
export type UserAuthenticationChangeEvent = CustomEvent<boolean>;
/**
 * UserPathHistory.
 * @remarks
 * APIs to track a user's path history. Includes the pathname, search and hash only.
 * This is not intended proxy or replace the History API.
 * @privateRemarks
 * Integrate with foundation analytics? Users could then simply call `user.trackPath` and we could engage analytics.
 * We should move analytics out of comms first.
 * @public
 */
export interface UserPathHistory {
    /**
     * Array of paths that have been tracked.
     */
    readonly pathHistory: string[];
    /**
     * Track current path.
     */
    trackPath(): string;
    /**
     * Clears path history.
     */
    clearPaths(): void;
    /**
     * May be used to return to a target path after authentication.
     * @returns The last stored path.
     */
    lastPath(): string | undefined;
}
/**
 * UserActionHistory.
 * @remarks
 * APIs to track a user's action history.
 * @privateRemarks
 * Integrate with foundation analytics? Users could then simply call `user.trackAction` and we could engage analytics.
 * We should move analytics out of comms first.
 * @public
 */
export interface UserActionHistory {
    /**
     * Array of actions that have been tracked.
     */
    readonly actionHistory: unknown[];
    /**
     * Track an action.
     */
    trackAction<TDetails = unknown>(details: TDetails): void;
    /**
     * Clears action history.
     */
    clearActions(): void;
    /**
     * @returns The last stored action.
     */
    lastAction(): unknown;
}
/**
 * User interface.
 * @privateRemarks
 * Providing `username` and `userName` for backwards compatability.
 * @public
 */
export interface User extends UserPathHistory, UserActionHistory {
    readonly isAuthenticated: boolean;
    readonly isAnonymous: boolean;
    readonly userKVs: {
        [key: string]: any;
    };
    readonly details: Details;
    readonly profile: string[];
    readonly sourceRef: string;
    readonly userName: string;
    readonly username: string;
    set(entity: Partial<LoginAck>): this;
    unset(): this;
    isAdmin(): boolean;
    isBanned(): boolean;
    hasPermission(permission: string | string[]): boolean;
    hasProfile(profile: string | string[]): boolean;
    addAuthenticationChangeListener(listener: Listener<UserAuthenticationChangeEvent>): void;
}
/**
 * Default User implementation.
 * @public
 */
export declare class DefaultUser implements User {
    protected config: UserConfig;
    pathHistory: string[];
    actionHistory: unknown[];
    isAuthenticated: boolean;
    userKVs: {
        [key: string]: any;
    };
    details: Details;
    permission: string[];
    profile: string[];
    sourceRef: string;
    userName: string;
    set(entity: Partial<LoginAck>): this;
    unset(): this;
    protected userNameChanged(): void;
    get username(): string;
    get isAnonymous(): boolean;
    isAdmin(): boolean;
    isBanned(): boolean;
    hasPermission(permission: string | string[]): boolean;
    hasProfile(profile: string | string[]): boolean;
    trackPath(): string;
    clearPaths(): void;
    lastPath(): string;
    trackAction<TDetails = unknown>(details: TDetails): void;
    clearActions(): void;
    lastAction(): unknown;
    protected hasKey(keys: string[], key: string | string[]): boolean;
    protected authenticationChangeListeners: import("@genesislcap/foundation-utils").Observer<UserAuthenticationChangeEvent>;
    addAuthenticationChangeListener(listener: Listener<UserAuthenticationChangeEvent>): () => void;
    protected isAuthenticatedChanged(): void;
}
/**
 * User DI key.
 *
 * @privateRemarks
 * Marked as internal to stop api-extractor becoming confused cross-linking tokens with the same name.
 *
 * @internal
 */
export declare const User: import("@microsoft/fast-foundation").InterfaceSymbol<User>;
/**
 * Gets the User from the DI container.
 * @remarks
 * A utility method for host applications that are not using our DI system directly.
 * @public
 */
export declare function getFromContainer(): User;
//# sourceMappingURL=user.d.ts.map