import { API_HOST, DEFAULT_ORGANISATION, DEFAULT_PASSWORD, DEFAULT_USER, SOCKET_EXT, } from '@genesislcap/foundation-utils';
import { css } from '@microsoft/fast-element';
import { DI, Registration } from '@microsoft/fast-foundation';
import { Route } from '@microsoft/fast-router';
import defaultBackgroundSrc from '../assets/bg.jpg';
import defaultLogoSrc from '../assets/genesis-logo-light.svg';
/**
 * @public
 */
export const defaultOrganisation = DEFAULT_ORGANISATION;
/**
 * @public
 */
export const defaultUser = DEFAULT_USER;
/**
 * @public
 */
export const defaultPassword = DEFAULT_PASSWORD;
/**
 * @public
 */
export const defaultApiHost = API_HOST;
/**
 * @public
 */
export const defaultSocketExt = SOCKET_EXT;
/**
 * @public
 */
export const hostEnv = location.host;
/**
 * @public
 */
export const hostUrl = API_HOST;
/**
 * @public
 */
export const Routes = {
    login: 'login',
    'request-account': 'request-account',
    'reset-password': 'reset-password',
    'forgot-password': 'forgot-password',
    verify: 'verify',
    protected: 'protected',
    'not-found': 'not-found',
};
/**
 * A utility to check if a pathname is a route of this micro frontend.
 *
 * @param pathname - A pathname string to check, typically location.pathname, which is used when a value is omitted.
 *
 * @public
 * @privateRemarks
 * I've just put this util in for now given there's some app level direction code in place which needs review.
 */
export function isMFRoute(pathname = location.pathname) {
    const valueParts = pathname.split('/');
    const routeIdx = pathname.charAt(0) === '/' ? 1 : 0;
    const route = valueParts[routeIdx];
    return !!Routes[route];
}
/**
 * A utility to get a sub route path taking the host path into account.
 *
 * @param routeName - The route name.
 * @param hostPath - The host path.
 *
 * @public
 */
export function getSubRoutePath(routeName, hostPath = '') {
    return hostPath !== '' ? `${hostPath}/${routeName}` : routeName;
}
/**
 * Default AuthConfig DI implementation.
 * @public
 */
export const defaultAuthConfig = {
    autoAuth: false,
    logo: css `
    content: url(${defaultLogoSrc});
  `,
    background: css `
    :host {
      background-image: url(${defaultBackgroundSrc});
    }
  `,
    omitRoutes: [],
    fields: {
        organisation: null,
        username: {
            label: 'Username',
            value: defaultUser,
            pattern: '^[a-zA-Z0-9.@_\\-]*$',
            title: 'Letters, numbers, and limited special characters @ _ - .',
        },
        email: {
            label: 'Email Address',
        },
        password: {
            label: 'Password',
            value: defaultPassword,
        },
    },
    hostPath: '',
    defaultRedirectUrl: '/protected',
    omitRedirectUrls: ['/not-found', '/not-permitted'],
    routeToUrl: (url) => {
        if (!!url && url !== Route.path.current) {
            /**
             * Uses the FASTRouter by default, however, host apps may override to integrate with their router of choice.
             * The FASTRouter invokes the History API and triggers a NavigationMessage to drive the view change.
             * TODO: We will need to test how unknown app level routers react to this default when not overriding.
             */
            Route.path.push(url);
        }
    },
    sso: {
        identityProvidersPath: 'gwf/sso/list',
    },
    feedbackDelay: 3000,
};
/**
 * AuthConfig DI key.
 *
 * @internal
 * @privateRemarks
 * Marked as internal to stop api-extractor becoming confused cross-linking tokens with the same name.
 */
export const AuthConfig = DI.createInterface((x) => x.instance(defaultAuthConfig));
/**
 * Configure the auth micro frontend settings.
 *
 * @remarks
 * This is primarily for general settings, rather than re-defining the element, static templating / style changes.
 *
 * @param config - A partial AuthConfig.
 *
 * @example Configuring auth on lazy load
 * ```ts
 * name: 'login',
 * path: 'login',
 * title: 'Login',
 * element: async () => {
 *   const { configure, define, defaultAuthConfig } = await import('@genesislcap/foundation-auth');
 *   configure({
 *     omitRoutes: ['request-account'],
 *     fields: {
 *       ...defaultAuthConfig.fields,
 *       organisation: {
 *         label: 'CompID',
 *       },
 *     },
 *     hostPath: 'login',
 *     defaultRedirectUrl: 'dashboard',
 *     logo: authLogo,
 *     background: authBG,
 *   });
 *   // You can import and return `Auth` directly here or re-define it completely via `define`.
 *   return define({
 *     name: `nexus-login`,
 *   });
 * },
 * layout: authLayout,
 * settings: { public: true },
 * childRouters: true,
 * ```
 *
 * @public
 * @privateRemarks
 * We can tailor the spreading logic after we put it to use. Keeping it simple for now.
 */
export function configure(config) {
    DI.getOrCreateDOMContainer().register(Registration.instance(AuthConfig, Object.assign(Object.assign({}, defaultAuthConfig), config)));
}
